// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

// For std::cout.
#ifndef iostream_h_included
#include <iostream>
#define iostream_h_included
#endif

#ifndef StreamUtil_h_included
#include "StreamUtil.h"
#endif

namespace {
  class ReportTimesFactory
    : public Factory
  {
  public:
    ReportTimesFactory ()
      : Factory ("ReportTimes")
      {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (ActionConfiguration ());
    }
    void pushTwo (Dynavec <Dynavec <FString> > &tableau,
		  const FString &label, const FString &value) const {
      tableau.push (Dynavec <FString> ());
      Dynavec <FString> &row = tableau[tableau.size()-1];
      row.push (label);
      row.push (value);
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      SP<ActionConfiguration> conf =
	dynamic_cast <ActionConfiguration *> (&*c);
      assert (conf);
      SP<TopLevel> top = conf->getTopLevel();
      Dynavec <Dynavec <FString> > tableau;
      pushTwo (tableau, "Event", "Milliseconds");
      TopLevel::Times times = top->getTimes ();
      pushTwo (tableau, "Drawing the last frame", times.lastDrawTime);
      pushTwo (tableau, "Last timestep", times.lastIntTime);
      pushTwo (tableau, "Timesteps for last frame", times.totalIntTime);
      pushTwo (tableau, "Slop",
	       times.interFrameTime -
	       times.lastDrawTime -
	       times.totalIntTime);
      pushTwo (tableau, "Time between frames", times.interFrameTime);
      StreamUtil::printTableau (std::cout, tableau);
      // No errors possible.
      return NEW (Action ());
    }
  };
  const bool useless =
  (FactoryTable::store ("Action", NEW (ReportTimesFactory ())),
   true);
} // namespace
