// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef ConstantRecursiveSlotValue_h_included
#include "ConstantRecursiveSlotValue.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef BadSlotValue_h_included
#include "BadSlotValue.h"
#endif

#ifndef iostream_h_included
#include <iostream>
#define iostream_h_included
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

ConstantRecursiveSlotValue::ConstantRecursiveSlotValue
(const FString &protocol, Factory *f)
  : RecursiveSlotValue (protocol, f, true)
{}

namespace {
  SP<Factory> safeGetFactory (const FString &protocol, const FString &factory) {
    SP<Factory> result = FactoryTable::load (protocol, factory);
    if (!result) {
      std::cerr << "Could not find factory " << factory << " in protocol "
	   << protocol << std::endl;
      abort ();
    }
    return result;
  }
}

ConstantRecursiveSlotValue::ConstantRecursiveSlotValue
(const FString &protocol, const FString &factory)
  : RecursiveSlotValue (protocol, safeGetFactory (protocol, factory), true)
{}

SP<SlotValue> ConstantRecursiveSlotValue::copy () const {
  return NEW (ConstantRecursiveSlotValue (*this));
}

FString ConstantRecursiveSlotValue::unParse () const {
  FString result = getConfiguration()->unParse ();
  if ("" == result) {
    return RecursiveSlotValue::unParse ();
  } else {
    return result;
  }
}

SP<ConstantRecursiveSlotValue> ConstantRecursiveSlotValue::getSlotValue
(Configuration * a_c, const FString &name)
{
  SP<Configuration> c = a_c;
  SP<SlotValue> slot = c->getSlot (name);
  ConstantRecursiveSlotValue *islot =
    dynamic_cast <ConstantRecursiveSlotValue *> (&*slot);
  assert (islot);
  return islot;
}

SP<const ConstantRecursiveSlotValue> ConstantRecursiveSlotValue::getSlotValue
(const Configuration *a_c, const FString &name)
{
  SP<const Configuration> c = a_c;
  SP<const SlotValue> slot = c->getSlot (name);
  const ConstantRecursiveSlotValue *islot =
    dynamic_cast <const ConstantRecursiveSlotValue *> (&*slot);
  assert (islot);
  return islot;
}

void ConstantRecursiveSlotValue::setSlotValue (SP<Configuration> c,
		   const FString &name,
		   SP<ConstantRecursiveSlotValue> rsv)
{
  SP<BadSlotValue> b = c->setSlot (name, &*rsv);
  assert (!b);
}

