// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef SpaceOrb_h_included
#include "SpaceOrb.h"
#endif

#ifndef InputDevicesFactory_h_included
#include "InputDevicesFactory.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef SimpleConfiguration_h_included
#include "SimpleConfiguration.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef Event_h_included
#include "Event.h"
#endif

#ifndef SpaceOrbEvent_h_included
#include "SpaceOrbEvent.h"
#endif

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef SpaceOrbConfiguration_h_included
#include "SpaceOrbConfiguration.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef LinuxJoystick_h_included
#include "LinuxJoystick.h"
#endif

#ifndef JoystickEvent_h_included
#include "JoystickEvent.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

#ifndef Quaternion_h_included
#include "Quaternion.h"
#endif

#ifndef InputDevices_h_included
#include "InputDevices.h"
#endif

namespace {
  FString spaceOrbName () {
    return "SpaceOrb 360";
  }
  typedef Event::Button Button;
  class LinuxSpaceOrb
    : public SpaceOrb
  {
    SP<LinuxJoystick> m_joystick;
    SP<SpaceOrbEvent> m_event;
    SP<const JoystickEvent> m_jsevent;
    SP<const SpaceOrbConfiguration> m_conf;
  public:
    LinuxSpaceOrb (SP <LinuxJoystick> lj)
      : m_joystick (lj)
    {
      assert (m_joystick->getName () == spaceOrbName ());
      m_event = dynamic_cast <SpaceOrbEvent *> (&*getVariableEvent ());
      assert (m_event);
      m_jsevent =
	dynamic_cast <const JoystickEvent *> (&*m_joystick->getEvent ());
      assert (m_jsevent);
      m_conf = dynamic_cast <const SpaceOrbConfiguration *> (&*getConf ());
      assert (m_conf);
    }
    FFloat compStick (FFloat num, FFloat stick) {
      if (num > stick) {
	return num - stick;
      } else if (num < -stick) {
	return num + stick;
      } else {
	return 0;
      }
    }
    bool readEvent () {
      if (m_joystick->readEvent ()) {
	Button b = m_jsevent->getButton ();
	Button newB;
	switch (b) {
	case 0: case 1: case 2: case 3: case 4: case 5:
	  newB = b + SpaceOrbEvent::A;
	  break;
	case 6:
	  newB = SpaceOrbEvent::RESET;
	  break;
	default:
	  newB = m_event->pointerMotionButton ();
	  if (b == m_jsevent->pointerMotionButton ()) {
	    // Several issues here:
	    // 1. I'll have to experiment to discover which physical ball
	    // motion is translated to which sign direction on which axis.
	    // 2. The user might want a different correspondence between
	    // Sporb motion and coordinate information depending on how he
	    // holds the sporb.  I'll ignore this issue until a user with a
	    // different preference from me complains.
	    // Doing a malloc call here.  Could hoist it into an instance
	    // variable.  I don't think there are enough events for this to
	    // matter.
	    Dynavec <FFloat> axes;
	    m_jsevent->getAxes (axes);
	    assert (6 == axes.size());
	    {
	      const FFloat tGain = m_conf->getTranslationalGain ();
	      const FFloat tStick = m_conf->getTranslationalStick ();
	      Vec3 t;
	      t [0] = compStick (axes [0] * tGain, tStick);
	      t [1] = compStick (axes [1] * tGain, tStick);
	      t [2] = compStick (axes [2] * tGain, tStick);
	      m_event->setTranslation (t);
	    }
	    {
	      const FFloat rGain = m_conf->getRotationalGain ();
	      const FFloat rStick = m_conf->getRotationalStick ();
	      Quaternion r;
	      r =
		Quaternion::makeRotation (compStick (axes [3] * rGain, rStick),
					  Vec3 (1, 0, 0)) *
		Quaternion::makeRotation (compStick (axes [4] * rGain, rStick),
					  Vec3 (0, 1, 0)) *
		Quaternion::makeRotation (compStick (axes [5] * rGain, rStick),
					  Vec3 (0, 0, 1));
	      m_event->setRotation (r);
	    }
	  } else {
	    die (FString ("Impossible button ") + (int) b +" from SpaceOrb.");
	    // No translation or rotation.
	    m_event->setRotation (Quaternion ());
	    m_event->setTranslation (Vec3 (0,0,0));
	  }
	  break;
	}
	m_event->setButton (newB, m_jsevent->isDown ());
	return true;
      } else {
	return false;
      }
    }
    int fdForSelect () const {
      return m_joystick->fdForSelect ();
    }
  };
  class LSOInputDevices
    : public InputDevices
  {
  public:
    void findDevices (Dynavec <SP <InputDevice> > &inputDevices) {
      inputDevices.extendTo (0);
      Dynavec <FString> devNames = LinuxJoystick::joystickDeviceNames ();
      for (int i = 0; i < devNames.size(); i++) {
	SP<LinuxJoystick> lj =
	  LinuxJoystick::openJoystickDevice (devNames [i]);
	if (lj && (spaceOrbName () == lj->getName ())) {
	  inputDevices.push (NEW (LinuxSpaceOrb (lj)));
	}
      }
    }
  };
  class LinuxSpaceOrbFactory
    : public InputDevicesFactory
  {
  public:
    LinuxSpaceOrbFactory ()
      : InputDevicesFactory ("LinuxSpaceOrbFactory")
    {}
    Dynavec <SP <InputDevicesFactory> > inferiorDeviceFactories () const {
      Dynavec <SP <InputDevicesFactory> > result;
      SP<InputDevicesFactory> linuxJoy =
	dynamic_cast <InputDevicesFactory *>
	(&*FactoryTable::load ("InputDevicesFactory", "LinuxJoystickFactory"));
      assert (linuxJoy);
      result.push (linuxJoy);
      return result;
    }
    SP<Configuration> defaultConfiguration () const {
      return NEW (SimpleConfiguration ());
    }
    SP<Configurable> makeIt (SP<Configuration> conf) const {
      (void) conf;
      return NEW (LSOInputDevices ());
    }
  };
  static const bool useless =
  (FactoryTable::store ("InputDevicesFactory", NEW (LinuxSpaceOrbFactory ())),
   true);
};
