// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef TypedFactory_h_included
#include "TypedFactory.h"
#endif

#ifndef ChordMouseDragConfiguration_h_included
#include "ChordMouseDragConfiguration.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef MouseDragBindingConfiguration_h_included
#include "MouseDragBindingConfiguration.h"
#endif

#ifndef MouseDragConfiguration_h_included
#include "MouseDragConfiguration.h"
#endif

#ifndef VecUtil_h_included
#include "VecUtil.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef Event_h_included
#include "Event.h"
#endif

#ifndef VectorSlotValue_h_included
#include "VectorSlotValue.h"
#endif

#ifndef ConfigurationSlotValue_h_included
#include "ConfigurationSlotValue.h"
#endif

#ifndef KeyboardMouseEvent_h_included
#include "KeyboardMouseEvent.h"
#endif

namespace {
  class ChordMouseDragGrab
    : public TypedFactory <ActionConfiguration, Action>
  {
    mutable SP<ChordMouseDragConfiguration> m_allBindings;
    mutable SP<MouseDragBindingConfiguration> m_currentBinding;
    SP<RecursiveSlotValue> currentDragRSV () const {
      return m_currentBinding->getMouseDrag ();
    }
  public:
    ChordMouseDragGrab 
    (ChordMouseDragConfiguration *bindings) 
      : TypedFactory <ActionConfiguration, Action> ("ChordMouseDragGrab"),
        m_allBindings (bindings)
    {}
    SP<ActionConfiguration> typedDefaultConfiguration () const
    {
      return NEW (ActionConfiguration ());
    }
    SP<Action> makeIt (ActionConfiguration *conf) const {
      SP<Action> result;
      SP<TopLevel> top = m_allBindings->getTopLevel();
      const SP<const KeyboardMouseEvent> thisEvent =
        static_cast <const KeyboardMouseEvent *> (&*top->getEvent ());
      typedef Event::Button Button;
      Dynavec <Button> downButtons = thisEvent->downButtons();
      // partialBinding will become true if we observe that there is any
      // overlap at all between the set of presently pushed buttons and any
      // binding.  We will release the grab if partialBinding is false after
      // the loop.
      bool partialBinding = false;
      // After the loop below, bestBinding will be non-null if any binding has
      // all of the required buttons down.  If multiple bindings match, then it
      // will be the binding with the most matching buttons.
      SP<MouseDragBindingConfiguration> bestBinding;
      const SP<VectorSlotValue> vsv = &*(m_allBindings->getBindings());
      for (int i = 0; i < vsv->size(); i++) {
        const SP<ConfigurationSlotValue> csv =
          static_cast <ConfigurationSlotValue *> (&*((*vsv)[i]));
        const SP<MouseDragBindingConfiguration> mdbc =
          static_cast <MouseDragBindingConfiguration *>
          (csv->getConfiguration());
        Dynavec <Button> boundButtons = mdbc->getShifts ();
        if (!partialBinding) {
          for (int i = 0; i < downButtons.size(); i++) {
            if (VecUtil::find (downButtons [i], boundButtons)) {
              partialBinding = true;
              break;
            }
          }
        }
        if (VecUtil::subset (boundButtons, downButtons)) {
          // A binding matched.  Is it the best one?
          if (!bestBinding ||
              (bestBinding->getShiftsVSV()->size () < boundButtons.size())) {
            bestBinding = mdbc;
          }
        }
      }
      if (bestBinding != m_currentBinding) {
        if (m_currentBinding) {
          // End the current drag.
          result =
            MouseDragConfiguration::endDrag (thisEvent, currentDragRSV());
        }
        m_currentBinding = bestBinding;
        if (m_currentBinding) {
          // Start a new drag.
          result =
            MouseDragConfiguration::startDrag (thisEvent, currentDragRSV(),
                                               top);
        }
        // bestBinding and m_currentBinding were different, so both cannot be
        // null.
        assert (result);
      } else {
        if (m_currentBinding) {
          // Tell the drag about any mouse motion.
          result =
            MouseDragConfiguration::mouseMoved (thisEvent, currentDragRSV());
        }
      }
      if (partialBinding) {
        // Regrab.
        SP<RecursiveSlotValue> grabRSV =
          NEW (RecursiveSlotValue ("KeyboardMouseAction", this));
        grabRSV->setConfiguration (conf);
        top->grab (grabRSV);
      } else {
        // Delegate to the default action.
        static_cast <ActionConfiguration *>
          (&*(m_allBindings->getDefaultAction()->getConfiguration()))
          ->setTopLevel (top);
        result =
          static_cast <Action *>
          (&*(m_allBindings->getDefaultAction()->makeIt ()));
      }
      return result;
    }
  };
  class ChordMouseDrag
    : public TypedFactory <ChordMouseDragConfiguration, Action>
  {
  public:
    ChordMouseDrag ()
      : TypedFactory <ChordMouseDragConfiguration, Action> ("ChordMouseDrag")
    {}
    SP<ChordMouseDragConfiguration> typedDefaultConfiguration () const {
      return NEW (ChordMouseDragConfiguration ());
    }
    SP<Action> makeIt (ChordMouseDragConfiguration *c) const {
      // Don't inline the computation of cmdg into the call to makeIt, since we
      // need somebody to hold the reference while makeIt is being called.
      SP<ChordMouseDragGrab> cmdg = NEW (ChordMouseDragGrab (c));
      return cmdg->makeIt (c);
    }
  };
  const bool useless =
  (FactoryTable::store ("KeyboardMouseAction", NEW (ChordMouseDrag ())),
   false);
}; // namespace
