#ifndef MouseDragBindingConfiguration_h_included
#define MouseDragBindingConfiguration_h_included

// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef Event_h_included
#include "Event.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

class RecursiveSlotValue;
class VectorSlotValue;
class MouseDragBindingConfiguration
// One mouse drag binding.  It has a list of shift keys and a
// RecursiveSlotValue with a mouse drag to do when the mouse is dragged with
// the shift keys down.  If you want to have the binding apply to a particular
// mouse button, include that button in the list of shift keys.  This is used
// in ChordMouseDragConfiguration.
  : public Configuration
{
public:
  typedef Event::Button Button;
  MouseDragBindingConfiguration ();
  MouseDragBindingConfiguration (const MouseDragBindingConfiguration &tc);
  SP<Configuration> copy () const;
  bool hasTheSameTypeAs (const Configuration *c) const;

  Dynavec <Button> getShifts () const;
  void setShifts (const Dynavec <Button> &v);
  // setShift is like calling setShifts with a dynavec of just one button.
  void setShift (Button b);
  SP<const VectorSlotValue> getShiftsVSV () const;
  SP<VectorSlotValue> getShiftsVSV ();
  void setShiftsVSV (SP<VectorSlotValue> vsv);
  // getMouseDrag returns a RecursiveSlotValue with a MouseDrag inside that is
  // the drag to do when mouse motion happens with the right set of shift
  // keys.  The default is NullMouseDrag, which does nothing.
  SP<const RecursiveSlotValue> getMouseDrag () const;
  SP<RecursiveSlotValue> getMouseDrag ();
};

#endif
