// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef TypedFactory_h_included
#include "TypedFactory.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef iostream_h_included
#include <iostream>
#define iostream_h_included
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef Event_h_included
#include "Event.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef KeyboardMouseEvent_h_included
#include "KeyboardMouseEvent.h"
#endif

#ifndef ChordMouseDragConfiguration_h_included
#include "ChordMouseDragConfiguration.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

namespace {
  
  // This class implements the scenario where you hit a keyboard key and then
  // drag the mouse to do something.  PrefixMouseDrag should be bound to the
  // keyboard key.  It should be configured to have the desired bindings.

  // Unfortunately for the simplicity of the code we are given the keypress
  // that got us started as well as the following key release.  We have no use
  // for this, but we have to wait for them to be over before we start
  // ChordMouseDrag, since it isn't ChordMouseDrag's scope to do anything with
  // them.  The plan is to ignore the initial keypress, then we ignore
  // events until we get a button going down after the initial key press, then
  // we dispatch that button-down to ChordMouseDrag with our own configuration.

  // We never register the BideTime factory.  It is used directly by
  // PrefixMouseDrag.
  class BideTime
    : public TypedFactory <ActionConfiguration, Action>
  {
    // Instead of carrying state information in the configuration, we carry it
    // in the factory.  This works because unlike most other factories, this
    // factory is constructed each time it is used, instead of being
    // constructed once and then put in the FactoryTable and then being used
    // over and over.
    const SP<ChordMouseDragConfiguration> m_cmdc;
  public:
    BideTime (ChordMouseDragConfiguration *cmdc)
      : TypedFactory <ActionConfiguration, Action> ("BideTime"),
      m_cmdc (cmdc)
    {}
    SP<ActionConfiguration> typedDefaultConfiguration () const {
      return NEW (ActionConfiguration ());
    }
    SP<Action> makeIt (ActionConfiguration *conf) const {
      SP<const Event> event = conf->getTopLevel()->getEvent ();
#ifndef NDEBUG
      std::cout << "Biding time; down is " << event->isDown ()
           << " and button is "
           << event->unParseButton (event->getButton ()) << std::endl;
#endif
      if (event->isDown ()) {
        // Invoke ChordMouseDrag.
        std::cout << "BideTime is invoking ChordMouseDrag." << std::endl;
        Action *a = dynamic_cast <Action *>
          (&*FactoryTable::load ("Action", "ChordMouseDrag")->
           makeIt (&*m_cmdc));
        assert (a);
        return a;
      } else {
        std::cout << "Waiting more." << std::endl;
        SP<RecursiveSlotValue> rsv =
          NEW (RecursiveSlotValue ("Action", this));
        // The default configuration is fine, since our configuration carries
        // no useful information to us, so there is no need to call
        // rsv->setConfiguration(); 
        conf->getTopLevel()->grab (rsv);
        return NEW (Action ());
      }
    }
  };

  class PrefixMouseDrag
    : public TypedFactory <ChordMouseDragConfiguration, Action>
  {
  public:
    PrefixMouseDrag ()
      : TypedFactory <ChordMouseDragConfiguration, Action> ("PrefixMouseDrag")
    {}
    SP<ChordMouseDragConfiguration> typedDefaultConfiguration () const {
      return NEW (ChordMouseDragConfiguration ());
    }
    SP<Action> makeIt (ChordMouseDragConfiguration *conf) const {
      std::cout << "PrefixMouseDrag is beginning to bide time." << std::endl;
      SP<RecursiveSlotValue> rsv =
        NEW (RecursiveSlotValue ("KeyboardMouseAction",
                                 NEW (BideTime (conf))));
      conf->getTopLevel()->grab (rsv);
      return NEW (Action ());
    }
  };
  const bool useless =
  (FactoryTable::store ("KeyboardMouseAction", NEW (PrefixMouseDrag ())),
   false);
}
