// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef BouncingBall_h_included
#include "BouncingBall.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef BouncingBallConfiguration_h_included
#include "BouncingBallConfiguration.h"
#endif

#ifndef BouncingBall_h_included
#include "BouncingBall.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef Color_h_included
#include "Color.h"
#endif

namespace {
  class BlueBouncingBallFactory : public Factory {
  public:
    BlueBouncingBallFactory () : Factory ("BlueBouncingBall")
    {}
    SP<Configuration> defaultConfiguration () const {
      SP<BouncingBallConfiguration> result = NEW (BouncingBallConfiguration());
      result->setVisibleRadius (1.0);
      result->setMaterial (Color (0.0, 0.0, 1.0, 1.0));
      result->setStiffness (10);
      result->setMass (8);
      return &*result;
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      BouncingBallConfiguration *conf =
	dynamic_cast <BouncingBallConfiguration *> (&*c);
      assert (conf);
      return NEW (BouncingBall (conf));
    }
  };
  
  static const bool useless =
  (FactoryTable::store ("BlueBouncingBall", NEW (BlueBouncingBallFactory ())),
   FactoryTable::registerSubclass ("BlueBouncingBall", "BouncingBall"),
   true);

}
