// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef BoringBallConfiguration_h_included
#include "BoringBallConfiguration.h"
#endif

#ifndef BoundingSphere_h_included
#include "BoundingSphere.h"
#endif

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef Ball_h_included
#include "Ball.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef BoundingBox_h_included
#include "BoundingBox.h"
#endif

#ifndef BoringBall_h_included
#include "BoringBall.h"
#endif

BoringBall::BoringBall (SP<BoringBallConfiguration> b) : Ball (&*b)
{}

bool BoringBall::exempt () const {
  return true;
}

bool BoringBall::isPhysicsBounded () const {
  return true;
}

BoundingSphere BoringBall::physicsBoundingSphere (const FFloat *state) const
{
  // No physics, so zero radius bounding sphere.  This still will
  // give interactions with other objects that overlap the center of
  // the BoringSphere; oh well.  
  return BoundingSphere (0, Vec3 (state));
}

bool BoringBall::physicsIntersect (const FFloat *state,
				   const BoundingBox &box)
  const
{
  (void) state; (void) box;
  return false;
}

bool BoringBall::isInstanceOf (const char *classId) const {
  // We avoid a jump by using | instead of || here.
  return (staticClassId == classId) | (Ball::staticClassId == classId);
}

void BoringBall::plausibleState (Dynavec <FFloat> &result) const {
  result.extendTo (0);
  result.extendTo (stateSize (), 0);
}

FFloat BoringBall::physicsBoundingSphereRadius () const {
  return 0;
}

const char *const BoringBall::staticClassId = "BoringBall";

const char *BoringBall::classId () const {
  return BoringBall::staticClassId;
}

namespace {

  class BoringBallFactory
    : public Factory
  {
  public:
    BoringBallFactory () : Factory ("BoringBall")
    {}
    SP<Configuration> defaultConfiguration () const {
      SP<BoringBallConfiguration> result = NEW (BoringBallConfiguration());
      return &*result;
    }
    SP<Configurable> makeIt (SP<Configuration> c) const {
      BoringBallConfiguration *conf = dynamic_cast <BoringBallConfiguration *> (&*c);
      assert (conf);
      return NEW (BoringBall (conf));
    }
  };
  
  static const bool useless =
  (FactoryTable::store ("Ball", NEW (BoringBallFactory ())),
   true);
};
