// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef NullObject_h_included
#include "NullObject.h"
#endif

#ifndef SimpleConfiguration_h_included
#include "SimpleConfiguration.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef BoundingBox_h_included
#include "BoundingBox.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

#ifndef TheObjectDrawerFactory_h_included
#include "TheObjectDrawerFactory.h"
#endif

#ifndef Quaternion_h_included
#include "Quaternion.h"
#endif

NullObject::NullObject ()
  : PhysicsObject ("Null", NEW (SimpleConfiguration ()))
{}

int NullObject::stateSize () const {
  return 0;
}

bool NullObject::physicsIntersect (const FFloat *state, const BoundingBox &box)
  const {
  (void) state; (void) box;
  return false;
}

void NullObject::plausibleState (Dynavec <FFloat> &result) const {
  result.extendTo (stateSize ());
}

Vec3 NullObject::boundingSphereCenter (const FFloat *state) const {
  (void) state;
  return Vec3 (0,0,0);
}

FFloat NullObject::physicsBoundingSphereRadius () const {
  return 0;
}

bool NullObject::isPhysicsBounded () const {
  return true;
}

void NullObject::translate (FFloat *state, const Vec3 &v) const {
  (void) state; (void) v;
  // Do nothing.
}

void NullObject::addVelocity (FFloat *state, const Vec3 &v) const {
  (void) state; (void) v;
  // Do nothing.
}

void NullObject::rotate (FFloat *state, const Quaternion &q) const {
  (void) state; (void) q;
  // Do nothing.
}

bool NullObject::exempt () const {
  return true;
}

const char *const NullObject::staticClassId = "NullObject";

const char *NullObject::classId () const {
  return staticClassId;
}

bool NullObject::isInstanceOf (const char *const classId) const {
  return staticClassId == classId;
}

bool NullObject::isSelectable () const {
  return false;
}

namespace {
  static const bool useless =
  (TheObjectDrawerFactory::registerObjectDrawer ("Null"),
   true);
}
