// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef NullObject_h_included
#define NullObject_h_included

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

class BoundingBox;

// An object with no physics, no state information.  Take a subclass of this
// and connect links to it to represent a set of PhysicsObjects fairly
// efficiently, while dealing automatically with updating the set when things
// are deleted.
class NullObject
  : public PhysicsObject
{
public:
  NullObject ();
  int stateSize () const;
  bool physicsIntersect (const FFloat *state, const BoundingBox &box)
    const;
  void plausibleState (Dynavec <FFloat> &result) const;
  Vec3 boundingSphereCenter (const FFloat *state) const;
  FFloat physicsBoundingSphereRadius () const;
  bool isPhysicsBounded () const;
  void translate (FFloat *state, const Vec3 &v) const;
  void addVelocity (FFloat *state, const Vec3 &v) const;
  void rotate (FFloat *state, const Quaternion &q) const;
  bool exempt () const;
  // The default implementation for doPhysics does nothing, which is
  // appropriate here, so don't define a new doPhysics here.
  const char *classId ()const;
  static const char *const staticClassId;
  bool isInstanceOf (const char *const classId) const;
  // Next one returns false.
  bool isSelectable () const;
};

#endif
