// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef PhysicsObjectInfo_h_included
#define PhysicsObjectInfo_h_included

#ifndef myassert_h_included
#include "myassert.h"
#endif

class PhysicsObject;

// This is just the information we need to do physics on an object.
struct PhysicsObjectInfo {
  const PhysicsObject * m_object;
  // The first index in the state vector for the object.
  int m_stateIndex;
  // The unique index in the list of objects for the object.
  int m_objectIndex;
  PhysicsObjectInfo (const PhysicsObject * const object,
                     int stateIndex, int objectIndex)
    : m_object (object), m_stateIndex (stateIndex), m_objectIndex (objectIndex)
  {}
  PhysicsObjectInfo () {}
  inline bool operator== (const PhysicsObjectInfo &other) const {
#ifndef NDEBUG
    if (m_objectIndex == other.m_objectIndex) {
      assert (m_object == other.m_object);
      assert (m_stateIndex == other.m_stateIndex);
    }
#endif
    return m_objectIndex == other.m_objectIndex;
  }
  inline bool operator!= (const PhysicsObjectInfo &other) const {
    return !(*this == other);
  }
  inline const PhysicsObjectInfo & operator= (const PhysicsObjectInfo &poi) {
    m_object = poi.m_object;
    m_stateIndex = poi.m_stateIndex;
    m_objectIndex = poi.m_objectIndex;
    return poi;
  }
};


#endif
