// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef myassert_h_included
#include "myassert.h"
#endif

// For strerror.
#ifndef  string_h_included
#include <string.h>
#define string_h_included
#endif

// For errno.
#ifndef errno_h_included
#include <errno.h>
#define errno_h_included
#endif

#ifndef iostream_h_included
#include <iostream>
#define iostream_h_included
#endif

#ifndef fstream_h_included
#include <fstream>
#define fstream_h_included
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef FileSceneLoaderConfiguration_h_included
#include "FileSceneLoaderConfiguration.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

#ifndef AtomFactory_h_included
#include "AtomFactory.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef FileSceneLoader_h_included
#include "FileSceneLoader.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef SelectionManager_h_included
#include "SelectionManager.h"
#endif

#ifndef StubBrennerAtomConfiguration_h_included
#include "StubBrennerAtomConfiguration.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

namespace {
  // A SceneLoader that reads the coord.d files that are the input and output
  // of Brenner's MD fortran distribution.  Has to be called DSceneLoader
  // because of the ".d", so tomFileSceneLoader can use this at the
  // appropriate time.
  class DSceneLoader
    : public FileSceneLoader
  {
    SP<Factory> m_f;
    SP<StubBrennerAtomConfiguration> m_bac;
    SP<AtomConfiguration> m_ac;
    SP<SceneGraph> m_sg;
    Vec3 m_translation;
    Dynavec <SP<PhysicsObject> > m_atoms;
    Dynavec <Dynavec <FFloat> > m_initialStates;
    void makeAtomLine (const FString &line, const int expectedNumber) {
      FString error;
      int lineNumber = line.toInt (0, &error);
      if (expectedNumber != lineNumber) {
	setProblem ("Expected atom number "+FString (expectedNumber)+
		    " but got "+ lineNumber);
	// But keep parsing.
      }
      int atomicNo = line.toInt (5, &error);
      const FFloat x = line.toFloat (11, &error);
      const FFloat y = line.toFloat (31, &error);
      const FFloat z = line.toFloat (51, &error);
      StubBrennerAtomConfiguration::MotionFlag motion =
	(StubBrennerAtomConfiguration::MotionFlag) line.toInt (71, &error);
      if (error.size()) {
	setProblem ("Could not parse the position line " + line + "\n" +
                    error);
      }
      if (!error.size()) {
	m_ac->setSymbolNumber (atomicNo);
        if (m_bac) {
          m_bac->setMotionFlag (motion);
        }
	SP<Configurable> obj = m_f->makeIt (&*m_ac);
        SP<PhysicsObject> atom = dynamic_cast <PhysicsObject *> (&*obj);
	assert (atom);
        Dynavec <FFloat> state;
	atom->plausibleState (state);
	atom->translate (state, m_translation);
	atom->translate (state, Vec3 (x, y, z));
        m_atoms.push (atom);
        m_initialStates.push (state);
      }
    }
    void readVelocity (const FString &line, const int expectedNumber, const
                       FFloat timestep) {
      FString error;
      int lineNumber = line.toInt (0, &error);
      if (expectedNumber != lineNumber) {
	setProblem ("Expected atom number "+FString (expectedNumber)+
		    " in the velocity list but got "+ lineNumber);
	// But keep parsing.
      }
      const FFloat x = line.toFloat (6, &error);
      const FFloat y = line.toFloat (26, &error);
      const FFloat z = line.toFloat (46, &error);
      if (error.size ()) {
        setProblem ("Could not parse the velocity line "+line+"\n"+error);
      } else {
        // The velocities stored in the file are the true velocities times the
        // timestep, so we have to divide by the timestep here.
        m_atoms[lineNumber-1]->addVelocity (m_initialStates[lineNumber-1],
                                            Vec3 (x, y, z)/timestep);
      }
    }
  public:
    DSceneLoader (SP<FileSceneLoaderConfiguration> slc) {
      m_f = FactoryTable::load ("Atom", "BrennerAtom");
      if (!m_f) {
        m_f = FactoryTable::load ("Atom", "StubBrennerAtom");
      }
      assert (m_f);
      m_bac =
	dynamic_cast <StubBrennerAtomConfiguration *>
	(&*m_f->defaultConfiguration ());
      m_ac = dynamic_cast <AtomConfiguration *>
        (&*m_f->defaultConfiguration ());
      assert (m_ac);
      m_sg = slc->getTopLevel ()->getSceneGraph ();
      assert (m_sg);
      m_translation = slc->getWhere ();
      FString fileName = slc->getFileName ();
      std::ifstream stream (&*fileName);
      if (!stream.is_open ()) {
	setProblem ("Could not open " + fileName + " for reading: "
		    + strerror (errno));
	return;
      }
      FString line;
      if (!isProblem ()) {
	stream >> line;
	if (stream.eof ()) {
	  setProblem ("Premature eof");
	}
      }
      if (!isProblem ()) {
	std::cout << "Scene title is " << line << std::endl;
	stream >> line;
	if (stream.eof ()) {
	  setProblem ("Premature eof");
	}
      }
      // Silly compiler thinks that if atoms isn't initialized here, it might
      // be used before it is initialized.  This isn't really true.  The
      // problem is that the compiler doesn't know that isProblem () can't
      // become false after it is true.
      int atoms = 0;
      if (!isProblem ()) {
	FString error = "";
	atoms = line.toInt (0, &error);
	if (error.size()) {
	  setProblem ("Failed to parse number of atoms: "+error);
	} else if (atoms < 0) {
	  setProblem ("Negative number " + FString (atoms) + " of atoms");
	}
      }
      if (!isProblem ()) {
	// Third line has elapsed time and timestep.
	stream >> line;
        if (stream.eof ()) {
          setProblem ("Premature eof");
        }
      }
      FFloat timestep = 0;
      if (!isProblem ()) {
        FString error = "";
        timestep = line.toFloat (22, &error);
        if (error.size ()) {
          setProblem ("Failed to parse timestep: "+error);
        }
      }
      if (!isProblem ()) {
	// Fourth line has periodic box; don't care.
	stream >> line; 
	if (stream.eof ()) {
	  setProblem ("Premature eof");
	}
      }
      m_atoms.extendTo (0);
      m_initialStates.extendTo (0);
      for (int i = 1; i <= atoms && !isProblem(); i++) {
	if (stream.eof ()) {
	  setProblem ("Premature eof");
	} else {
	  stream >> line;
	  makeAtomLine (line, i);
	}
      }
      for (int i = 1; i <= atoms && !isProblem(); i++) {
	if (stream.eof ()) {
	  setProblem ("Premature eof");
	} else {
	  stream >> line;
	  readVelocity (line, i, timestep);
	}
      }

      // If we opened it, it's important to close even if there were problems.
      stream.close ();
      // If there was a partial read, then put the junk read so far into the
      // scene anyway.  This will help figure out where we failed.
      {
        Dynavec <int> indices;
        for (int i = 0; i < m_atoms.size(); i++) {
          indices.push (m_sg->addObject (m_atoms[i], m_initialStates[i]));
        }
        if (slc->getSelect ()) {
          // If asked to select everything we added, do it even if there were
          // problems.
          Dynavec <int> selection;
          for (int i = 0; i < indices.size(); i++) {
            const int s = indices [i];
            if (-1 != s && m_sg->object (s)->isSelectable ()) {
              selection.push (s);
            }
          }
          SelectionManager::setSelection (m_sg, selection);
        }
      }
      if (isProblem()) {
	setProblem (getProblem () + " in " + fileName);
      }
    }
  };
  class DSceneLoaderFactory
    : public Factory
  {
  public:
    DSceneLoaderFactory ()
      : Factory ("DSceneLoader")
    {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (FileSceneLoaderConfiguration ());
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      FileSceneLoaderConfiguration *conf =
	dynamic_cast <FileSceneLoaderConfiguration *> (&*c);
      assert (conf);
      return NEW (DSceneLoader (conf));
    }
  };
  const bool useless =
  (FactoryTable::store ("FileSceneLoader", NEW (DSceneLoaderFactory ())),
   true);
};
