// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifdef IncrementalStorageSceneConfiguration_h_included
#warn Missing external include guard for IncrementalStorageSceneConfiguration
#endif

#ifndef IncrementalStorageSceneConfiguration_h_included
#define IncrementalStorageSceneConfiguration_h_included

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef SceneGraphConfiguration_h_included
#include "SceneGraphConfiguration.h"
#endif

#ifndef BadSlotValue_h_included
#include "BadSlotValue.h"
#endif

class IncrementalStorageSceneConfiguration : public SceneGraphConfiguration {
  const FString m_slotName;
public:
  IncrementalStorageSceneConfiguration ()
    : m_slotName ("Incremental Storage Factory")
  {
    initializeSlot (m_slotName,
		    NEW (RecursiveSlotValue
			 (FactoryTable::load ("IncrementalStorage",
					      "ArrayStorage"))));
  }
  void setIncrementalStorageFactory (Factory *f) {
    SP<BadSlotValue> b = setSlot (m_slotName, NEW (RecursiveSlotValue (f)));
    assert (!b);
  }
  Factory *getIncrementalStorageFactory () const {
    SP<RecursiveSlotValue> slot =
      dynamic_cast <RecursiveSlotValue *> (getSlot (m_slotName));
    assert (slot);
    return slot->getFactory ();
  }
  Configuration *getIncrementalStorageConfiguration () const {
    SP<RecursiveSlotValue> slot =
      dynamic_cast <RecursiveSlotValue *> (getSlot (m_slotName));
    assert (slot);
    return slot->getConfiguration ();
  }
  SP<Configuration> copy () const;
  bool hasTheSameTypeAs (const Configuration *c) const;
};

#endif
