// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#include "TestSceneConfiguration.h"

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef PositiveIntSlotValue_h_included
#include "PositiveIntSlotValue.h"
#endif

#ifndef PositiveFloatSlotValue_h_included
#include "PositiveFloatSlotValue.h"
#endif

#ifndef Vec3SlotValue_h_included
#include "Vec3SlotValue.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

// Don't have a static variable with a string, it looks like a memory leak.
// So use functions instead.
#ifndef SlotValue_h_included
#include "SlotValue.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

static const FString objName () {
  return "What to Build";
}

static const FString cubeSize () {
  return "How many objects along edges of cube";
}

static const FString cubeSpace () {
  return "Distance between objects";
}

TestSceneConfiguration::TestSceneConfiguration ()
{
  initializeSlot (objName (),
		  NEW (RecursiveSlotValue ("PhysicsObject",
					   "RedBouncingBall")));
  ref ();
  setWhere (Vec3 (0, 0, 15));
  deref ();
  const int redcubeSize =
#ifdef NDEBUG
    8
#else
    3
#endif
    ;
  initializeSlot (cubeSize (), NEW (PositiveIntSlotValue (redcubeSize)));
  initializeSlot (cubeSpace (), NEW (PositiveFloatSlotValue (1.0)));
}

SP<PhysicsObject> TestSceneConfiguration::makeIt () const {
  SP<Configurable> r = getRecursiveSlotValue ()->makeIt ();
  SP<PhysicsObject> result = dynamic_cast <PhysicsObject *> (&*r);
  assert (result);
  return result;
}

void TestSceneConfiguration::setRecursiveSlotValue (SP<RecursiveSlotValue> r) {
  RecursiveSlotValue::setSlotValue (this, objName (), r);
}

SP<RecursiveSlotValue> TestSceneConfiguration::getRecursiveSlotValue () {
  return RecursiveSlotValue::getSlotValue (SP<Configuration> (this),
					   objName ());
}

SP<const RecursiveSlotValue> TestSceneConfiguration::getRecursiveSlotValue ()
  const
{
  return RecursiveSlotValue::getSlotValue (this, objName ());
}

int TestSceneConfiguration::getCubeSize () const {
  return PositiveIntSlotValue::getSlot (this, cubeSize ());
}

void TestSceneConfiguration::setCubeSize (int s) {
  PositiveIntSlotValue::setSlot (this, cubeSize (), s);
}

FFloat TestSceneConfiguration::getCubeSpace () const {
  return PositiveFloatSlotValue::getSlot (this, cubeSpace ());
}

void TestSceneConfiguration::setCubeSpace (FFloat s) {
  PositiveFloatSlotValue::setSlot (this, cubeSpace (), s);
}

SP<Configuration> TestSceneConfiguration::copy () const {
  return NEW (TestSceneConfiguration (*this));
}
bool TestSceneConfiguration::hasTheSameTypeAs (const Configuration *c)
  const
{
  return 0 != dynamic_cast <const TestSceneConfiguration *> (c);
}

FString TestSceneConfiguration::unParse () const {
  return getRecursiveSlotValue()->unParse();
}
