// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef XBallDrawerConfiguration_h_included
#include "XBallDrawerConfiguration.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef PositiveFloatSlotValue_h_included
#include "PositiveFloatSlotValue.h"
#endif

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

#ifndef LinkManager_h_included
#include "LinkManager.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

#ifndef BoringAtom_h_included
#include "BoringAtom.h"
#endif

#ifndef FloatUtil_h_included
#include "FloatUtil.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef XObjectDrawer_h_included
#include "XObjectDrawer.h"
#endif

#ifndef BallConfiguration_h_included
#include "BallConfiguration.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

#ifndef XLineDrawer_h_included
#include "XLineDrawer.h"
#endif

#ifndef TypedFactory_h_included
#include "TypedFactory.h"
#endif

#ifndef Color_h_included
#include "Color.h"
#endif

#ifndef AtomConfiguration_h_included
#include "AtomConfiguration.h"
#endif

#ifndef TheObjectDrawerFactory_h_included
#include "TheObjectDrawerFactory.h"
#endif

#ifndef AtomInfo_h_included
#include "AtomInfo.h"
#endif

#ifndef DepthClipInfo_h_included
#include "DepthClipInfo.h"
#endif

#ifndef BoundingSphere_h_included
#include "BoundingSphere.h"
#endif

#ifndef BoolSlotValue_h_included
#include "BoolSlotValue.h"
#endif

namespace {
  class XWireFrameDrawerConfiguration
    : public XBallDrawerConfiguration
  {
    static FString clickRadiusName () {
      return "Radius for mouse clicks and atoms without bonds";
    }
    static FString minFadeName () {
      return "Remaining intensity for wires near background";
    }
    static FString alwaysBallName () {
      return "Always draw balls";
    }
  public:
    XWireFrameDrawerConfiguration () {
      initializeSlot (clickRadiusName (), NEW (PositiveFloatSlotValue (0.5)));
      initializeSlot (minFadeName (), NEW (PositiveFloatSlotValue (0.25)));
      initializeSlot (alwaysBallName (), NEW (BoolSlotValue (false)));
    }
    FFloat getClickRadius () const {
      return PositiveFloatSlotValue::getSlot (this, clickRadiusName ());
    }
    void setClickRadius (FFloat f) {
      PositiveFloatSlotValue::setSlot (this, clickRadiusName (), f);
    }
    FFloat getMinFade () const {
      return PositiveFloatSlotValue::getSlot (this, minFadeName ());
    }
    void setMinFade (FFloat f) {
      assert (0 < f && f <= 1);
      PositiveFloatSlotValue::setSlot (this, minFadeName (), f);
    }
    bool getAlwaysBall () const {
      return BoolSlotValue::getSlot (this, alwaysBallName());
    }
    void setAlwaysBall (bool b) {
      BoolSlotValue::setSlot (this, alwaysBallName(), b);
    }
    SP<Configuration> copy () const {
      return NEW (XWireFrameDrawerConfiguration (*this));
    }
    bool hasTheSameTypeAs (const Configuration *c) const {
      return 0 != dynamic_cast <const XWireFrameDrawerConfiguration *> (c);
    }
  };
  SP<const XObjectDrawer> genBallDrawer (const Color &c,
					 XWireFrameDrawerConfiguration *conf)
  {
    SP<XBallDrawerConfiguration> xbc =
      dynamic_cast <XBallDrawerConfiguration *> (&*(conf->copy ()));
    assert (xbc);
    // Copy should have copied the physics object configuration too.
    assert (xbc->getPhysicsObjectConfiguration () !=
	    conf->getPhysicsObjectConfiguration ());
    SP<BallConfiguration> bc = NEW (BallConfiguration ());
    // The radius of the ball, when we draw it, is our click radius.
    bc->setVisibleRadius (conf->getClickRadius ());
    bc->setMaterial (c);
    xbc->setPhysicsObjectConfiguration (&*bc);
    SP<const Configurable> result1 =
      &* (FactoryTable::load ("ObjectDrawer", "XBallDrawer")->makeIt (&*xbc));
    assert (result1);
    // The atom factory is supposed to prevent us from creating very many of
    // these, so doing a dynamic_cast each time we create is okay.
    SP<const XObjectDrawer> result2 =
      dynamic_cast <const XObjectDrawer *> (&*result1);
    return result2;
  }
  class XWireFrameDrawer
    : public XObjectDrawer
  {
    // Delegate to the XBallDrawer instead of being a subclass of it, since
    // that's cleaner and the class isn't public.
    const SP<const XObjectDrawer> m_ballDrawer;
    const FFloat m_clickRadius;
    const Color m_color;
    const FFloat m_minFade;
    const bool m_alwaysDrawBalls;
  public:
    XWireFrameDrawer (const Color &c, XWireFrameDrawerConfiguration *conf)
      : m_ballDrawer (genBallDrawer (c, conf)),
	m_clickRadius (conf->getClickRadius ()),
	m_color (c),
	m_minFade (conf->getMinFade ()),
	m_alwaysDrawBalls (conf->getAlwaysBall())
    {}
    bool intersect (const FFloat *state,
		    const Vec3 &v1, const Vec3 &v2, Vec3 *result,
		    int objectIndex, const SceneGraph *sg)
      const
    {
      // We might try to support clicking on the bonds here, but not today.
      return m_ballDrawer->intersect (state, v1, v2, result,
				      objectIndex, sg);
    }
    bool graphicsIntersect (const FFloat *state, const BoundingBox &box,
			    int objectIndex, const SceneGraph *sg)
      const {
      return m_ballDrawer->graphicsIntersect (state, box, objectIndex, sg);
    }
    bool intersect (const FFloat *state, const Vec3 &v,
		    int objectIndex, const SceneGraph *sg) const {
      return m_ballDrawer->intersect (state, v, objectIndex, sg);
    }
    // Bug: have a bunch of wireframe atoms near each other, one with a really
    // long bond.  The old scheme used the graphicsBoundingSphere to determine
    // what you could select, so the outcome was that no matter where in the
    // vicinity you clicked, you select the one with the long bond.  The
    // solution is to have a separate method for determining the bounding
    // sphere radius for the purpose of doing selections.
    // FIXME This will break down once we have objects with truly non-spherical
    // sensitive areas, like bouncing cubes.  The right primitive would say
    // whether an orthogonal transformation applied to an x,y,z box intersects
    // the object.  Yuck, complicated.  Another option is to give up on
    // selecting a frustrum and only allow selecting the one object under the
    // mouse.
    BoundingSphere selectionBoundingSphere (const FFloat *state,
					    int objectIndex,
					    const SceneGraph *sg)
      const
    {
      (void) objectIndex; (void) sg;
      return BoundingSphere (m_clickRadius, boundingSphereCenter (state));
    }

    FFloat graphicsBoundingSphereRadius (int objectIndex,
					const SceneGraph *sg) const {
      // Return the maximum of m_clickRadius and half the distance to the
      // furthest bonded atom.
      const Vec3 &myCenter =
	BoringAtom::center (sg->objectState (objectIndex));
      int count;
      const int *links;
      sg->getLinkManager ()->getLinks (objectIndex, count, links);
      FFloat maxBondLengthSquared = 0;
      for (int i = 0; i < count; i++) {
	const int link = links [i];
	if (-1 != link) {
	  const PhysicsObject *po = sg->object (link);
	  if (po->isInstanceOf (BoringAtom::staticClassId)) {
	    const Vec3 &otherCenter =
	      BoringAtom::center (sg->objectState (link));
	    maxBondLengthSquared =
	      FloatUtil::max (maxBondLengthSquared,
			      (otherCenter - myCenter).lengthSquared ());
	  }
	}
      }
      if (maxBondLengthSquared > 4 * FloatUtil::sqr (m_clickRadius)) {
	return FloatUtil::sqrt (maxBondLengthSquared)/2;
      } else {
	return m_clickRadius;
      }
    }
    Vec3 boundingSphereCenter (const FFloat *state) const {
      return m_ballDrawer-> boundingSphereCenter (state);
    }
    // Draw lines from the present atom (with center specified by state) to its
    // neighbors (with neighbor numbers specified by the first count integers
    // found at links, and the meanings of the neighbor numbers given by sg).
    // vo is the frame to draw on.  
    // dci should be maintained with information about the number of pixels at
    // each depth.
    // finished is an input indicating which depth bins are finished (so maybe
    // we could skip drawing them).
    // FIXME Why don't I glue together dci, finished, and vo into an object
    // so I don't have to pass around so much cruft?
    void drawLines (const FFloat *state, const int *links, int count,
		    const SceneGraph *sg,
		    const Canvas *vo, const DepthClipInfo &dci,
		    const BinSet &finished) const {
      // Draw some lines
      // Note that where differs from where2 because where has already been
      // translated so its depth is measured from the near Z plane. 
      const Vec3i where2 = vo->transformWorldToLHScreen
	(BoringAtom::center (state)).truncateToVec3i ();
      for (int i = 0; i < count; i++) {
	const int link = links [i];
	if (-1 != link &&
	    sg->object (link)->isInstanceOf (BoringAtom::staticClassId)) {
	  Vec3i otherCenter =
	    vo->transformWorldToLHScreen
	    (BoringAtom::center (sg->objectState (link))).truncateToVec3i();
	  Vec3i otherEnd = (where2 + otherCenter) / 2;
	  XLineDrawer::drawFadingLine (vo, where2, otherEnd, m_color,
				       dci, finished, m_minFade);
	}
      }
    }
    void draw (const FFloat *state, const Canvas *vo,
	       const Vec3i &where, const DepthClipInfo &dci,
	       const BinSet &finished, int objectIndex,
	       const SceneGraph *sg) const {
      const LinkManager *lm = sg->getLinkManager ();
      int count;
      const int *links;
      lm->getLinks (objectIndex, count, links);
      if (m_alwaysDrawBalls) {
	m_ballDrawer->draw (state, vo, where, dci, finished,
			    objectIndex, sg);
	drawLines (state, links, count, sg, vo, dci, finished);
      } else {	
	bool foundBond = false;
	for (int i = 0; i < count; i++) {
	  const int link = links [i];
	  if (-1 != link &&
	      sg->object (link)->isInstanceOf (BoringAtom::staticClassId)) {
	    foundBond = true;
	    break;
	  }
	}
	if (!foundBond) {
	  m_ballDrawer->draw (state, vo, where, dci, finished,
			      objectIndex, sg);
	} else {
	  drawLines (state, links, count, sg, vo, dci, finished);
	}
      }
    }
    bool isVisible () const {
      return true;
    }
  };
  class XWireFrameDrawerFactory
    : public TypedFactory <XWireFrameDrawerConfiguration, XWireFrameDrawer>
  {
  public:
    XWireFrameDrawerFactory ()
      :
      TypedFactory <XWireFrameDrawerConfiguration, XWireFrameDrawer> ("XWireFrameDrawer")
    {}
    SP<XWireFrameDrawerConfiguration> typedDefaultConfiguration () const {
      return NEW (XWireFrameDrawerConfiguration ());
    }
    SP<XWireFrameDrawer> makeIt (XWireFrameDrawerConfiguration *c) const {
      SP<AtomConfiguration> ac = 
	dynamic_cast <AtomConfiguration *>
	(&*c->getPhysicsObjectConfiguration ());
      assert (ac);
      const int num = ac->getSymbolNumber ();
      return NEW (XWireFrameDrawer (AtomInfo::CPKColor (num), c));
    }
  };
  class XBallWireFrameDrawerFactory
    : public TypedFactory <XWireFrameDrawerConfiguration, XWireFrameDrawer>
  {
  public:
    XBallWireFrameDrawerFactory ()
      :
      TypedFactory <XWireFrameDrawerConfiguration, XWireFrameDrawer>
    ("XBallWireFrameDrawer")
    {}
    SP<XWireFrameDrawerConfiguration> typedDefaultConfiguration () const {
      SP<XWireFrameDrawerConfiguration> result =
	NEW (XWireFrameDrawerConfiguration ());
      result->setAlwaysBall (true);
      return result;
    }
    SP<XWireFrameDrawer> makeIt (XWireFrameDrawerConfiguration *c) const {
      SP<AtomConfiguration> ac = 
	dynamic_cast <AtomConfiguration *>
	(&*c->getPhysicsObjectConfiguration ());
      assert (ac);
      const int num = ac->getSymbolNumber ();
      return NEW (XWireFrameDrawer (AtomInfo::CPKColor (num), c));
    }
  };
  const bool useless =
  (AtomConfiguration::registerDrawer ("WireFrame", "Wire Frame"),
   AtomConfiguration::registerDrawer ("BallWireFrame", "Ball and Wire"),
   TheObjectDrawerFactory::registerObjectDrawer ("WireFrame"),
   TheObjectDrawerFactory::registerObjectDrawer ("BallWireFrame"),
   FactoryTable::store ("ObjectDrawer", NEW (XWireFrameDrawerFactory ())),
   FactoryTable::store ("ObjectDrawer", NEW (XBallWireFrameDrawerFactory ())),
   true);

}
