#!/usr/local/sbin/perl -w
use strict;
sub usage () {
    die << "EOM";
Usage: $0 < xmol.d > xmol.d.new
Normalize cosmetic features of an xmol.d file.
The situation is that C likes to print floating point numbers as
5.00000000000e-01 and Fortran likes to print the same number as
0.50000000000E+00 so you go nuts trying to see if they're printing the
same numbers.  This program reads an xmol.d file and prints out the
floating point numbers the way it wants to, so if you filter both
the C and the Fortran output through this program, the cosmetic
differences will be eliminated.
EOM
}

# Print a bunch of integers four characters wide, each followed by a space.
sub p {
    my $num;
    foreach $num (@_) {
	(printf "%4d ", $num) || die "Error on print: $!";
    }
}

# Print a float 20 characters wide, followed by a space.
sub f {
    my $num;
    foreach $num (@_) {
	(printf "%20f", $num) || die "Error on printf: $!";
    }
}

sub main () {
    for (;;) {
	my $uselessline = <STDIN>;
	last unless defined $uselessline;
	print "filename eliminated by normalization\n";
	my $countline = <STDIN>;
	$countline =~ m!^\s*(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s*$!
	    || die "Bad line $countline";
	my $atomcount = $1;
	p ($atomcount, $2, $3, $4); print "\n";
	my $timeline = <STDIN>;
	$timeline =~ m!^\s*([^\s]+)\s+([^\s]+)\s*$!
	    || die "Bad line $timeline";
	f ($1, $2); print "\n";
	my $boxline = <STDIN>;
	$boxline =~ m!^\s*([^\s]+)\s*([^\s]+)\s+([^\s]+)\s*$!
	    || die "Bad line $boxline";
	f ($1, $2, $3); print "\n";
	my $i;
	for ($i = 1; $i <= $atomcount; $i++) {
	    my $line = <STDIN>;
	    $line =~ m!^\s*(\d+)\s+(\d+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s*$!
		|| die "Bad line $line";
	    my ($lineno, $atom, $x, $y, $z) = ($1, $2, $3, $4, $5);
	    die "Bad line number in $line" unless $lineno == $i;
	    p ($lineno); p($atom); f ($x, $y, $z); print "\n";
	}
    }
}

main ();
