# Fungimol - an extensible system for designing atomic-scale objects.
# Copyright (C) 2000 Tim Freeman
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with this library in the file COPYING.txt; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place -
# Suite 330, Boston, MA 02111-1307, USA
#
# The author can be reached by email at tim@fungible.com, or by
# paper mail at:
#
# Tim Freeman
# 381 North Fernwood Circle
# Sunnyvale, CA 94085-3059
#

# This is the makefile that should be edited at install time.
# This used to be inlined into the top level Makefile, but that didn't
# work because a make in a subdirectory ought to have the same
# configuration as a top level make.  The issue then was the
# DOUBLE_PRECISION flag, but any of this configuration
# information could have been the issue.  Tim Freeman 16 Nov 2000.

# The c++ compiler to use.  This is overridden to be simply "g++" when
# compiling from RPM, since we should expect them to have the right
# c++ version on their path rather than installed in the same place I
# happen to have it.
ifndef CPLUSPLUS
   CPLUSPLUS := g++
endif
export CPLUSPLUS

# The directory to put the tar files into for "make tar".
TARHOME:=/home/tim/fungimol/tarhome
export TARHOME

# The version number for Fungimol.
VERSION:=0.5.1
export VERSION

# The version number for brennermd.  
BRENNERMD_VERSION=0.1.3

# We don't export BRENNERMD_VERSION.  Instead, we pass it in by using
# BRENNERMAKE to do a build in the brenner directory.  Search for the
# definition of BRENNERMAKE below to find out more.  I (Tim Freeman
# as of 6 Jul 2000) don't remember why I treat this differently from
# the Fungimol version.

# Default path for plugins.  Some installation code assumes that this
# is just one directory, that is, it does not have any colons.
DEFAULT_FUNGIMOL_PATH:=/home/tim/brennerc/brennerc/fungimol
export DEFAULT_FUNGIMOL_PATH

# This gets set while making the RPM.  It gets prepended to all
# pathnames when we install, so it will be easy to clean up the
# installation later.  RPM will copy the files so that RPM_BUILD_ROOT
# before the copy gets mapped to / after the copy, so don't insert
# RPM_BUILD_ROOT into any source code before compiling.  It should be
# blank, or have a leading slash and no trailing slash.
ifndef RPM_BUILD_ROOT
   RPM_BUILD_ROOT:=
endif
export RPM_BUILD_ROOT

# Where to put plugins.
PLUGINHOME:=/usr/share/fungimol/plugins
export PLUGINHOME

# Where to install the binaries.
INSTALLHOME:=/usr/bin

# Where to install the documentation.  Beware that when we do an
# uninstall, this entire directory will be rm -rf'd, so don't use a
# directory that will have other stuff in it.  (This does not apply if
# you're using "rpm -e" to uninstall.)
DOCHOME:=/usr/doc/fungimol-$(VERSION)
export DOCHOME

# Where to put the include files.  Treated like DOCHOME when we do an
# uninstall, so don't use a directory that has anything else important
# in it.
INCLUDEHOME:=/usr/include/fungimol
export INCLUDEHOME

# Where the Fungimol executable will reside.  
# In the fullbuild and install targets, a recursive make is used to
# set this to INSTALLHOME.
ifndef EXEHOME
   EXEHOME:=$(shell pwd)/App/nodebug
endif
export EXEHOME

# Where sources for source RPM's typically reside.  The maximum RPM
# document says /usr/src/redhat, but on my system it's /usr/src/RPM.
# The trend seems to be for Red Hat to take their name off of RPM
# (presumably to make it a widely-accepted standard), so I'm guessing
# that /usr/src/RPM is right in the long term.
# As of 29 Jan 2003 and Red Hat 8.0, only /usr/src/redhat will work.
RPMHOME:=/usr/src/redhat

# Where to put the splines for Brenner's molecular dynamics program.
# Unfortunately this has to match strings in the source code for
# brennermd; we don't automatically edit that source code to match
# what appears here.
SPLINEHOME=/usr/share/brennermd
export SPLINEHOME

# At the moment, if you're using Brennerc and Fungimol together, you
# need DOUBLE_PRECISION turned on.
DOUBLE_PRECISION=yes
export DOUBLE_PRECISION

# Whether to use mcheck for debug builds.  Mcheck is a Linux package
# (that may also exist on other systems, for all I know) that tries to
# stop promptly when typical heap corruption bugs happen.
# If you have the header and it isn't in the usual place, you'll have
# to edit this makefile to set the flag to yes, and also add the right
# directory to the include path.
ifeq ($(wildcard /usr/include/mcheck.h),)
   MCHECK:=no
else
   MCHECK:=yes
endif
export MCHECK

# Whether to try to use Linux joystick support.
# If you have the header and it isn't in the usual place, you'll have
# to edit this to set it to yes anyway and probably edit this
# makefile elsewhere so it can find the header.
ifeq ($(wildcard /usr/include/linux/joystick.h),)
   LINUX_JOYSTICK:=no
else
   LINUX_JOYSTICK:=yes
endif
export LINUX_JOYSTICK

